##########################################
## ---- ---- ---- ---- ---- ---- ---- - ##
##        Compartimental model          ##  
##            Audrey Duval              ##                      
##        Last update 07-01-2019        ##
## ---- ---- ---- ---- ---- ---- ---- - ##
##########################################

rm(list=ls())
library(deSolve)
source("permutation_functions_light.R")
load("mat.day.RData")
load("entero_dataFrame.RData")


#----------------- FUNCTIONS -------------------#
nb_voisin_moyen<-function(ind,tmp){
  #return the average number of neighbour of ind with the contact matrix tmp   
  
  #first all ind neighbours (1-distance)
  IDs.list<-unique(c(as.character(tmp$from[which(as.character(tmp$to)==ind)]),as.character(tmp$to[which(as.character(tmp$from)==ind)])))
  
  #second neighbours of neighbours (2-distance)
  voisin<-c()
  for(g in IDs.list){
    IDs.list.gens<-unique(c(as.character(tmp$from[which(as.character(tmp$to)==g)]),as.character(tmp$to[which(as.character(tmp$from)==g)])))
    IDs.list.gens<-IDs.list.gens[which(IDs.list.gens != ind)]
    voisin<-c(voisin,IDs.list.gens)
  }
  voisin<-unique(voisin)

  return(length(voisin[which(substr(voisin,1,2)!="PE")]))
}


ligne_mat.day<-function(tmp,cat1,cat2){
  return(which((substr(as.character(tmp$from),1,2)==cat1 & substr(as.character(tmp$to),1,2)==cat2) | (substr(as.character(tmp$from),1,2)==cat2 & substr(as.character(tmp$to),1,2)==cat1)))
}

mean_voisin_scenario<-function(patientDeMat.day,mat.day){
  tab<-sapply(patientDeMat.day,function(x){sapply(seq(9,max(mat.day$week),by=1),function(y){nb_voisin_moyen(x,mat.day[which(mat.day$week==y),])})})
  tab[which(tab==0)]<-NA
  meanParPatients<-apply(tab,2,mean,na.rm=T)
  meanPatientsTOT<-mean(meanParPatients)
  return(meanPatientsTOT)
}

model_SC<-function(param_BR,c, bacterie, n,Tmax=25){
  B=param_BR[1]*c
  D=param_BR[4]/100
  G=1/param_BR[2]
  v=(1-param_BR[3])*(param_BR[4]/100)
  dt=1
  
  N=mean(patient_par_semaine)
  C0<-param_BR[5]*N
  S0<-(1-param_BR[5])*N
  I0=0
  
  Time=seq(9,Tmax,dt)
  Init.cond=c(S0,C0,I0) 
  param=list(B=B,G=G,N=N,v=v,D=D)
  
  # Execute
  result<- lsoda(Init.cond, Time, SC.deterministic, param)
  tab<- as.data.frame(result)
  # tab
  names(tab)=c("time","S","C","I")
  tab$prevalence<-round(tab$C/N,2)

  Inc=rep(0,nrow(tab))
  ICumul=tab$I
  Inc[1]=ICumul[1]
  for(j in 1: (length(Inc)-1)){
    Inc[j+1]=ICumul[j+1] - ICumul[j]
  }
  
  tab$Inc<-Inc
  return(tab)
}


SC.deterministic<-function(t,pop,param){
  
  #pop
  S<-pop[1]
  C<-pop[2]
  I<-pop[3]

  #param
  with(as.list(param),{
    dS <- -B*S*(C/N)-v*S+G*C
    dC <- B*S*(C/N)+ v*S -G*C
    dI <- B*S*(C/N)+ v*S 
    res<-c(dS, dC, dI)
    list(res)
  })
  
}

returnMePercentage<-function(param_EC, param_KP, meanPatientsTOT){
  
  #1) first scenario: remove 100% PA-PA CPIs
  without_PAPA<-ligne_mat.day(mat.day,"PA","PA")
  mat_scenario1<-mat.day[-without_PAPA,]
  individusDemat_scenario1<-unique(c(as.character(mat_scenario1$from),as.character(mat_scenario1$to)))
  patientDemat_scenario1<-individusDemat_scenario1[which(substr(individusDemat_scenario1,1,2)=="PA")]
  c_scenario1<-mean_voisin_scenario(patientDemat_scenario1,mat_scenario1)
  
  #2) second scenario: remove 100% PA-PE CPIs
  without_PAPE<-ligne_mat.day(mat.day,"PA","PE")
  mat_scenario2<-mat.day[-without_PAPE,]
  individusDemat_scenario2<-unique(c(as.character(mat_scenario2$from),as.character(mat_scenario2$to)))
  patientDemat_scenario2<-individusDemat_scenario2[which(substr(individusDemat_scenario2,1,2)=="PA")]
  c_scenario2<-mean_voisin_scenario(patientDemat_scenario2,mat_scenario2)
  
  
  #3) third scenario: remove 50% PA-PE CPIs
  without_PAPE<-sample(ligne_mat.day(mat.day,"PE","PA"),0.5*length(ligne_mat.day(mat.day,"PE","PA")))
  mat_scenario3<-mat.day[-without_PAPE,]
  individusDemat_scenario3<-unique(c(as.character(mat_scenario3$from),as.character(mat_scenario3$to)))
  patientDemat_scenario3<-individusDemat_scenario3[which(substr(individusDemat_scenario3,1,2)=="PA")]
  c_scenario3<-mean_voisin_scenario(patientDemat_scenario3,mat_scenario3)
  
  
  #4) fourth scenario: remove 50% PA-PA CPIs
  without_PAPA50<-sample(ligne_mat.day(mat.day,"PA","PA"),0.5*length(ligne_mat.day(mat.day,"PA","PA")))
  mat_scenario4<-mat.day[-without_PAPA50,]
  individusDemat_scenario4<-unique(c(as.character(mat_scenario4$from),as.character(mat_scenario4$to)))
  patientDemat_scenario4<-individusDemat_scenario3[which(substr(individusDemat_scenario4,1,2)=="PA")]
  c_scenario4<-mean_voisin_scenario(patientDemat_scenario4,mat_scenario4)
  
  m1<-model_SC(param_EC,meanPatientsTOT,"Escherichia coli","sans mesure",Tmax=25)
  m2<-model_SC(param_KP,meanPatientsTOT,"Klebsiella pneumoniae","sans mesure",Tmax=25)
  m3<-model_SC(param_EC,c_scenario1,"Escherichia coli", "100% PA-PA",Tmax=25)
  m4<-model_SC(param_KP,c_scenario1,"Klebsiella pneumoniae", "100% PA-PA",Tmax=25)
  m5<-model_SC(param_EC,c_scenario2,"Escherichia coli","100% PA-PE",Tmax=25)
  m6<-model_SC(param_KP,c_scenario2,"Klebsiella pneumoniae","100% PA-PE",Tmax=25)
  m7<-model_SC(param_EC,c_scenario3,"Escherichia coli","50% PA-PE",Tmax=25)
  m8<-model_SC(param_KP,c_scenario3,"Klebsiella pneumoniae","50% PA-PE",Tmax=25)
  m9<-model_SC(param_EC,c_scenario4,"Escherichia coli","50% PA-PA",Tmax=25)
  m10<-model_SC(param_KP,c_scenario4,"Klebsiella pneumoniae","50% PA-PA",Tmax=25)
  
  sc1_EC<-(max(m1$I)-max(m3$I))/max(m1$I)
  sc2_EC<-(max(m1$I)-max(m5$I))/max(m1$I)
  sc3_EC<-(max(m1$I)-max(m7$I))/max(m1$I)
  sc4_EC<-(max(m1$I)-max(m9$I))/max(m1$I)
  
  sc1_KP<-(max(m2$I)-max(m4$I))/max(m2$I)
  sc2_KP<-(max(m2$I)-max(m6$I))/max(m2$I)
  sc3_KP<-(max(m2$I)-max(m8$I))/max(m2$I)
  sc4_KP<-(max(m2$I)-max(m10$I))/max(m2$I)
  
  
  scenario_EC<-c(sc1_EC,sc4_EC,sc2_EC,sc3_EC)
  scenario_KP<-c(sc1_KP,sc4_KP,sc2_KP,sc3_KP)
  
  legend_scenario<-rep(c(rep("Interventions \n based on \n patient \n isolation",2),rep("Interventions \n based on \n staff \nhand hygiene",2)))
  cat_pourcent<-rep(c("100%","50%","100%","50%"))
  legend_scenario2<-c()
  for(i in 1:length(legend_scenario)){
    legend_scenario2<-c(legend_scenario2,paste(legend_scenario[i],"\n(",cat_pourcent[i],")"))
  }
  
  
  df<-data.frame(pourcentage=c(scenario_EC,scenario_KP),scenario=legend_scenario2,bacteria=c(rep("Escherichia coli",4),rep("Klebsiella pneumoniae",4)),stringsAsFactors = F)
  
  return(df)
  
}


#--------------- MAIN -----------------#
mat.day<-unsymetrize.contact.matrix(mat.day)
individusDeMat.day<-unique(c(as.character(mat.day$from),as.character(mat.day$to)))
patientDeMat.day<-individusDeMat.day[which(substr(individusDeMat.day,1,2)=="PA")]

mat.day$week<-as.numeric(strftime(mat.day$day,"%W"))-17
entero.all$week<-as.numeric(strftime(entero.all$date_prl,"%W"))-17

#swabbed people by week
gens_prlvt<-list()
for(i in unique(entero.all$week)){
  gens_prlvt[[i]]<-unique(as.character(entero.all$calc_ident[which(entero.all$week==i)]))
}
unlist(lapply(gens_prlvt,length))

#people with CPIs by week
gens_mat<-list()
for(i in unique(mat.day$week)){
  tmp<-mat.day[which(mat.day$week==i),]
  gens_mat[[i]]<-unique(c(as.character(tmp$from[which(substr(tmp$from,1,2)=="PA")]),as.character(tmp$to[which(substr(tmp$to,1,2)=="PA")])))
}
unlist(lapply(gens_mat,length))

#People swabbed + CPIs by week
commune<-list()
for(i in 9:25){
  commune[[as.character(i)]]<-unique(union(gens_prlvt[[i]],gens_mat[[i]]))
}
commune2<-lapply(commune,function(x){return(x[which(x %in% individusDeMat.day)])}) 
patient_par_semaine<-unlist(lapply(commune2,length))

# Determine the number of distinct CPIs/week at CPI distance < or = to 2
meanPatientsTOT<-mean_voisin_scenario(patientDeMat.day, mat.day)

#% of individual the first week
patient1ereSemaine<-patient_par_semaine[1]
patientColoniseEC<-length(unique(entero.all$calc_ident[which(entero.all$week==9 & entero.all$BLSE==1 & entero.all$ident=="Escherichia coli")]))
patientColoniseKP<-length(unique(entero.all$calc_ident[which(entero.all$week==9 & entero.all$BLSE==1 & entero.all$ident=="Klebsiella pneumoniae")]))

p0_EC<-patientColoniseEC/patient1ereSemaine
p0_KP<-patientColoniseKP/patient1ereSemaine


#Weekly % of incidence (from description tab)
incidEC<-1.96
incidKP<-1.15

#Number of weekly patients
N<-mean(patient_par_semaine)

#Initialization with prevalence
p0_EC<-(11.5/100)
p0_KP<-(3.73/100)

#duration
dureeEC<-11.5/incidEC
dureeKP<-3.73/incidKP

#All parameters
##EC
tauEC <- 0.51 #from permutation analysis
GEC<- 1/dureeEC
DEC<- incidEC/100
sEC<- 1-(11.5/100)
cEC<- (11.5/100)
vEC<- (1-tauEC)*(incidEC/100)
##KP
tauKP <- 0.75 #from permutation analysis
GKP<- 1/dureeKP
DKP<- incidKP/100
sKP<- (1-(3.73/100))
cKP<- (3.73/100)
vKP<- (1-tauKP)*(incidKP/100)

pEC<- (1/meanPatientsTOT) * ( (GEC/sEC) - (vEC/cEC))
pKP<- (1/meanPatientsTOT) * ( (GKP/sKP) - (vKP/cKP) )

param_EC<-c(pEC,dureeEC,tauEC,incidEC,p0_EC)
param_KP<-c(pKP,dureeKP,tauKP,incidKP,p0_KP)

df=returnMePercentage(param_EC, param_KP, meanPatientsTOT)

#PLOT
legend_scenario<-rep(c(rep("Interventions \n based on \n patient \n isolation",2),rep("Interventions \n based on \n staff \nhand hygiene",2)))
cat_pourcent<-rep(c("100%","50%","100%","50%"))
legend_scenario2<-c()
for(i in 1:length(legend_scenario)){
  legend_scenario2<-c(legend_scenario2,paste(legend_scenario[i],"\n(",cat_pourcent[i],")"))
}

ggplot(data=df, aes(x=factor(scenario,levels = unique(legend_scenario2)), y=pourcentage*100, fill=bacteria)) + geom_bar(stat="identity", position=position_dodge()) + theme_bw() + ylab("Cumulative incidence reduction (%)") + xlab("") + theme(legend.title = element_blank())  + scale_fill_brewer(labels = c("ESBL-EC", "ESBL-KP"), palette = "Set1")+theme(axis.text=element_text(size=16, colour="black"),axis.title=element_text(size=16, colour="black"),legend.text=element_text(size=16))

