############################################
## ---- ---- ---- ---- ---- ---- ---- --- ##
##    ESBL code permutation FUNCTIONS     ##             
##             Audrey DUVAL               ##
##        Last Updated 07 01 2019         ##
## ---- ---- ---- ---- ---- ---- ---- --- ##
############################################

#require(RecordLinkage)
require(statnet)
require(network)
require(networkDynamic)
require(ergm.userterms)
require(ergm)
require(tergm)
require(ndtv)
require(plotrix)
require(survival)
require(sna)
require(RColorBrewer)
require(ggplot2)
#require(vioplot)
require(gplots)
require(lme4)
require(varComp)
require(rmeta)
require(animation)
#require(intervals)
require(intergraph)
#require(igraph)
require(msm)
require(MASS)


SupSimilar<-function(col.entero){
  col.entero2<-col.entero
  for(i in unique(col.entero$calc_ident)){
    tmp<-col.entero[which(col.entero$calc_ident==i),]
    #we check for multiple isolate 
    if(length(names(which(table(tmp$date_prl)>1)))>0){
      for(d in names(which(table(tmp$date_prl)>1))){
        tmp2<-tmp[which(tmp$date_prl==d),]
        tab_combi<-combn(tmp2$profile.antibio,2)
        vect_identique<-c()
        for(c in 1:ncol(tab_combi)){
          seq1<-transfo_atb(tab_combi[1,c])
          seq2<-transfo_atb(tab_combi[2,c])
          s<-score(seq1,seq2)
          if(FALSE %in% sapply(s,function(x){if(x>1){return(FALSE)}else{return(TRUE)}})){
            next
          }else{
            vect_identique<-c(vect_identique,tab_combi[,c])
          }
        }
        vect_identique<-unique(vect_identique)
        col.entero2$profile.antibio[which(col.entero2$calc_ident==i & col.entero2$date_prl==d & (col.entero2$profile.antibio %in% vect_identique))]<-vect_identique[1]
      }
    }
  }
  return(unique(col.entero2))
}


transfo_atb<-function(chaine){
  r=2
  s=0
  i=1
  u=4
  y=0
  chaine<-unlist(strsplit(chaine,""))
  chaine[which(chaine=="R")]<-r
  chaine[which(chaine=="S")]<-s
  chaine[which(chaine=="I")]<-i
  chaine[which(chaine=="U")]<-u
  chaine[which(chaine=="Y")]<-y
  return(chaine)
}

score<-function(x,y){
  return(abs(as.numeric(x)-as.numeric(y)))
}


select.episodes <- function(what="Escherichia coli", with="BLSE", lissage=FALSE) {
  #find incident-colonization episodes of what. With correspond to the ESBL resistance (BLSE or clean). Scenario options can be specific to the species or not. Lissage check for two previous negative swab if TRUE or just one negative swab if FALSE.  
  
  if (!(what %in% c("Escherichia coli","Klebsiella pneumoniae"))) stop("Incorrect \'what\' input (Escherichia coli, Klebsiella pneumoniae, all)")
  if (!(with %in% c("BLSE","clean"))) stop("Incorrect \'with\' input (BLSE, clean)")

  res <- NULL
  prl <- entero.all
  
  #colonization type
  if (with=="BLSE") {
    wanted.col <- 1
  }else {
    wanted.col <- 0
  }
  
  #For each patients, we find indicident-colonization episodes
  for (i in unique(as.character(prl$calc_ident))) {
    tmp <- subset(prl, prl$calc_ident==i)
    
    date<-unique(tmp$date_prl_posix)
    
    dt.min <- min(tmp$date_prl_posix)
    if(lissage){
      at.first <- subset(tmp, tmp$date_prl_posix <=  date[which(date==dt.min)+1])
    }else{
      at.first <- subset(tmp, tmp$date_prl_posix == dt.min)
    }
    
    nb.at.first <- dim(at.first)[1]
    
    if (dim(tmp)[1] == nb.at.first | length(date)==1) {
      next
    }
    
    # main counter
    t <- nb.at.first+1
    
    
    while (t <= dim(tmp)[1]) {
      
      # current swab
      current.date <- tmp$date_prl_posix[t]
      current.prl <- subset(tmp, tmp$date_prl_posix == current.date)
      nb.now <- dim(current.prl)[1]
      current.ident<-current.prl$ident
      
      #previous swab
      if(lissage){
        previous.date <- c(date[which(date==current.date)-1], date[which(date==current.date)-2])
        previous.prl <- subset(tmp, tmp$date_prl_posix %in% previous.date)
      }else{
        previous.date <- date[which(date==current.date)-1]
        previous.prl <- subset(tmp, tmp$date_prl_posix == previous.date)
      }
      
      nb.previous <- dim(previous.prl)[1]
      previous.ident<-previous.prl$ident

      #for each current swab we check the incident-colonization episode
      for (cur in 1:nb.now) {
        #we work with the (prlvt.current / prlvt.previous) couple
        tmp.res <- NULL
        if (as.numeric(current.prl$BLSE[cur]) != wanted.col) {
          #if we don't have an ESBL in the current swab we pass to the next line
          next
        }else{
            #we check incident-colonization with the species
            if(current.prl$ident[cur]!=what){
              #if we don't have what we are looking for, next
              next
            }else{
              
              if(!(1 %in% as.numeric(previous.prl$BLSE))){
                #if we don"t have any ESBL -> it's an acquisition
                tmp.res <- c(i, current.prl$date_prl[cur], as.character(current.prl$date_prl_posix[cur]), round(difftime(current.prl$date_prl_posix[cur], min(previous.date), units="days")),current.prl$ident[cur],current.prl$BLSE[cur],current.prl$profile.antibio[cur])
              }else{
                score_atb<-c()
                v<-rep(NA,nrow(previous.prl))
                names(v)<-previous.prl$ident
                for(l in 1:nrow(previous.prl)){
                  
                  seq1<-transfo_atb(current.prl$profile.antibio[cur])
                  seq2<-transfo_atb(previous.prl$profile.antibio[l])
                  s<-score(seq1,seq2)
                  #if the score > 1 then true
                  if(TRUE %in% sapply(s,function(x){if(x>1){return(TRUE)}else{return(FALSE)}})){
                    # we have antibiotics > 1 so seq2!=seq1
                    v[l]<-TRUE
                    
                  }else{
                    #NO atb > 1 so seq1=seq2
                    v[l]<-FALSE
                    
                  }
                }
                
                if(!(FALSE %in% v)){
                  # if we don't have any false inside v no seq1=seq2 -> acquisition
                  tmp.res <- c(i, current.prl$date_prl[cur], as.character(current.prl$date_prl_posix[cur]), round(difftime(current.prl$date_prl_posix[cur], min(previous.date), units="days")),current.prl$ident[cur],current.prl$BLSE[cur],current.prl$profile.antibio[cur])
                }else{
                  # else we have a false so an isolat have a similar sequence. We look for if the species is different 
                  if(!(what %in% names(v[which(v==FALSE)]))){
                    tmp.res <- c(i, current.prl$date_prl[cur], as.character(current.prl$date_prl_posix[cur]), round(difftime(current.prl$date_prl_posix[cur], min(previous.date), units="days")),current.prl$ident[cur],current.prl$BLSE[cur],current.prl$profile.antibio[cur])
                  }
                }
              }
            }
        }
        res <- rbind(res, tmp.res)
      }
      t <- t + nb.now
    }
  }
  #building the output
  if (!is.null(res)) {
    colnames(res) <- c("calc_ident", "date_prl", "date_prl_posix", "time_since_last_prlvt", "ident","BLSE","profile.antibio")
    res <- data.frame(res, row.names=(1:dim(res)[1]),stringsAsFactors = FALSE)
    res$date_prl_posix <- as.POSIXct(res$date_prl_posix)
    res$BLSE<-as.numeric(res$BLSE)
    res$ident<-as.character(res$ident)
    
  }
  return(res)
}



find.same.germ <- function(x, prlvt=entero.all, lag=7, delta.profile=0, shuffle=FALSE, pass=1, b=21, what="Escherichia coli",verbose=FALSE, ...) {
  # x: data.frame with all incident colonization episodes
  # prlvt: data.frame with all swabw
  # lag: nb of days
  # delta.profile: maximum allow for the susceptible profile
  # shuffle: randomisation status
  # pass: first or next shuffle 
  # verbose: debug
  #b: max permutations time
  # what for the species

  ncase <- dim(x)[1]
  res <- NULL
  resolved <- NULL
  orphan <- NULL
  
  if(is.null(ncase)){return(list(found=res, notfound=orphan, resolved=resolved))}
  
  if ((shuffle)) {prlvt.bak <- prlvt; shuff.prlvt <- list()}
  
  # Incident-colonization episodes loop
  for (i in 1:ncase) {
    c.ID <- as.character(x$calc_ident[i])
    c.date_prl <- as.Date(as.character(x$date_prl[i]), origin="1970-01-01")
    c.profile.antibio <- as.character(x$profile.antibio[i])
    c.blse<-as.numeric(x$BLSE[i])
    c.ident<-as.character(x$ident[i])
    
    if (verbose) {cat("Proceeding with line ", i, "/", ncase, " (ID: ", c.ID, ", date: ", as.character(c.date_prl), ").\n", sep="")}
    
    if ((shuffle) & (pass==1)) {
      # swab Permutations only columns relative to the isolat from swabbed people during the specific window 
      #colonization independance (loose of the ID/souche link -> loose CPI/infection link)
      cat("Warning: shuffle is set to TRUE. Colonization status will be randomly permuted.\n")
      prlvt <- prlvt.bak
      prlvt <- prlvt[(as.Date(prlvt$date_prl, origin="1970-01-01") >= (c.date_prl - b) & as.Date(prlvt$date_prl, origin="1970-01-01") < (c.date_prl) 
                      & as.character(prlvt$calc_ident) != c.ID), ]
      if (dim(prlvt)[1]>0) {
        ordr <- 1:dim(prlvt)[1]
        prlvt$cut <- as.numeric(difftime(c.date_prl, as.Date(as.character(prlvt$date_prl)), units="days"))
        prlvt$cut <- cut(prlvt$cut, breaks=c(1,seq(7,b,by=7)), include.lowest=TRUE)
        ordr <- as.numeric(unlist(tapply(ordr, prlvt$cut, function(x) sample(x))))
        prlvt[, c(4:6)] <- prlvt[ordr, c(4:6)]
        prlvt <- prlvt[, -which(names(prlvt) == "cut")]
      }
      
      if (!(paste(as.character(c.date_prl), c.ID, sep=".") %in% names(shuff.prlvt))) {shuff.prlvt[[paste(as.character(c.date_prl), c.ID, sep=".")]] <- prlvt}
    }
    
    if ((shuffle) & (pass>1)) prlvt <- prlvt.bak[[paste(as.character(c.date_prl), c.ID, sep=".")]]
    
    tmp <- prlvt[(as.Date(as.character(prlvt$date_prl), origin="1970-01-01") >= (c.date_prl - lag) & as.Date(as.character(prlvt$date_prl), origin="1970-01-01") < (c.date_prl)
                  & as.numeric(prlvt$BLSE) == c.blse
                  & as.character(prlvt$ident) == c.ident
                  & as.character(prlvt$calc_ident) != c.ID
                  & !(is.na(prlvt$ident))), ]
    
    
    if (dim(tmp)[1] == 0) {warning("No other prelevements meet the requirements with the specified parameters."); orphan <-rbind(orphan, x[i, ]); next}
    stockage<-NULL
    t<-list()
    for(s in 1:nrow(tmp)){
      seq1<-c.profile.antibio
      seq2<-tmp$profile.antibio[s]
      vecteur_distance<-score(transfo_atb(seq1),transfo_atb(seq2))
      t[[s]]<-vecteur_distance
      if(FALSE %in% sapply(vecteur_distance,function(x){if(x>1){return(FALSE)}else{return(TRUE)}})){  
        next
      }else{
        stockage<-rbind(stockage,tmp[s,])
        
      }
    }
    tmp<-stockage
    
    if (is.null(tmp)) {warning("No other prelevements meet the requirements with the specified parameters."); orphan <-rbind(orphan, x[i, ]); next}
    res <- rbind(res, cbind(c.ID, c.date_prl, tmp,c.profile.antibio, i))
    resolved <- rbind(resolved, cbind(x[i, ], i))
    
  }
  
  # OUTPUT
  if (!is.null(res)) {
    colnames(res) <- c("case_id", "case_date_prl", "calc_ident", "date_prl", "date_prl_posix" ,"ident","BLSE","profile.antibio", "service_pa_pe","profile.antibio.cas","group")
    res <- data.frame(res, row.names=(1:dim(res)[1]), stringsAsFactors=FALSE)
    res$case_id <- as.factor(as.character(res$case_id))
    res$case_date_prl <- as.Date(res$case_date_prl, origin="1970-01-01")
    res$calc_ident <- as.factor(as.character(res$calc_ident))
    res$date_prl <- as.Date(res$date_prl, origin="1970-01-01")
    res$date_prl_posix <- as.POSIXct(res$date_prl_posix)
    res$profile.antibio <- as.character(res$profile.antibio)
    res$profile.antibio.cas <- as.character(res$profile.antibio.cas)
  }
  
  # resolved episodes
  if (!is.null(resolved)) {
    colnames(resolved) <- c(names(x), "group")
    resolved <- data.frame(resolved, row.names=(1:dim(resolved)[1]), stringsAsFactors=FALSE)
    resolved$calc_ident <- as.factor(as.character(resolved$calc_ident))
    resolved$date_prl <- as.Date(resolved$date_prl, origin="1970-01-01")
    resolved$date_prl_posix <- as.POSIXct(resolved$date_prl_posix)
    resolved$time_since_last_prlvt <- as.numeric(resolved$time_since_last_prlvt)
    resolved$profile.antibio <- as.character(resolved$profile.antibio)
    resolved$group <- as.numeric(resolved$group)
  }
  
  #orphan = no candidates found
  if (!is.null(orphan)) {
    orphan <- data.frame(orphan, row.names=(1:dim(orphan)[1]), stringsAsFactors=FALSE)
    orphan$calc_ident <- as.factor(as.character(orphan$calc_ident))
    orphan$date_prl <- as.Date(orphan$date_prl, origin="1970-01-01")
    orphan$date_prl_posix <- as.POSIXct(orphan$date_prl_posix)
    orphan$time_since_last_prlvt <- as.numeric(orphan$time_since_last_prlvt)
    orphan$profile.antibio <- as.character(orphan$profile.antibio)
  }
  
  if ((shuffle) & (pass==1)) return(list(found=res, notfound=orphan, resolved=resolved, prlvt=shuff.prlvt))
  else return(list(found=res, notfound=orphan, resolved=resolved))
}



find.closest.infector <- function (x, nw=mat.day, dur=0, lag=7,  keepall=FALSE, verbose=FALSE) {
  # x: data.frame candidates
  # nw: contact matrix
  # dt: date
  # dur: minimal duration (secondes)
  # lag: number of aggregated days
  # keepall: keep all possible infectors (TRUE) or just the closest (FALSE)
  # verbose: debug

  found <- x$found
  resolved <- x$resolved
  cases <- unique(found$group)
  ncases <- length(cases)
  res <- NULL
  orphan <- NULL
  
  if(is.null(found)){
    return(list(resolved=res, unresolved=orphan))
  }
  
  # loop on cases
  for (i in 1:ncases) {
    tmp <- found[as.character(found$group) == cases[i], ]
    c.ID <- as.character(tmp$case_id[1])
    c.date_prl <- as.Date(tmp$case_date_prl[1], origin="1970-01-01")
    
    if (verbose) {cat("Proceeding with line ", i, "/", ncases, " (ID: ", c.ID, ", date: ", as.character(c.date_prl), ", ", dim(tmp)[1], " ID(s) to test", ").\n", sep="")}
    
    tmp.res <- NULL

    for (l in 1:dim(tmp)[1]) {
      alter.ID <- as.character(tmp$calc_ident[l])
      dist <- distance.from.ID(from=c.ID, to=alter.ID, nw=nw, dt=c.date_prl, dur=dur, lag=lag)
      if (!is.null(dist)) {tmp.res <- rbind(tmp.res, c(c.ID, alter.ID, 
                                                       as.character(c.date_prl), 
                                                       dist[1],
                                                       dist[2],
                                                       as.character(admission.raw$service_pa_pe[as.character(admission.raw$calc_ident) == c.ID]),
                                                       as.character(admission.raw$service_pa_pe[as.character(admission.raw$calc_ident) == alter.ID]),
                                                       as.numeric(tmp$BLSE[l]),
                                                       as.character(tmp$profile.antibio[l]),
                                                       as.character(tmp$profile.antibio.cas[l]),
                                                       cases[i]))}
    }
    
    # closest candidate
    if (!is.null(tmp.res)) {
      if (!(keepall)) tmp.res <- tmp.res[tmp.res[, 4] == min(tmp.res[, 4]), ]
      res <- rbind(res, tmp.res)
    }else {orphan <- rbind(orphan, resolved[resolved$group == cases[i], -match("group", colnames(resolved))])}
  }
  
  # OUTPUT
  if (!is.null(res)) {
    colnames(res) <- c("case", "alter", "case_date_prl", "distance", "intermediate", "case_service", "alter_service","BLSE", "profile.antibio","profile.antibio.cas", "group")
    res <- data.frame(res, row.names=1:dim(res)[1], stringsAsFactors=FALSE)
    res$case <- factor(as.character(res$case))
    res$alter <- factor(as.character(res$alter))
    res$case_date_prl <- as.Date(res$case_date_prl, origin="1970-01-01")
    res$distance <- as.numeric(res$distance)
    res$intermediate <- as.character(res$intermediate)
    res$case_service <- as.character(res$case_service)
    res$alter_service <- as.character(res$alter_service)
    res$profile.antibio <- as.character(res$profile.antibio)
    res$group <- as.numeric(res$group)
  }
  
  if (!is.null(orphan)) {
    orphan <- data.frame(orphan, row.names=(1:dim(orphan)[1]), stringsAsFactors=FALSE)
    orphan$calc_ident <- as.factor(as.character(orphan$calc_ident))
    orphan$date_prl <- as.Date(orphan$date_prl, origin="1970-01-01")
    orphan$date_prl_posix <- as.POSIXct(orphan$date_prl_posix)
    orphan$time_since_last_prlvt <- as.numeric(orphan$time_since_last_prlvt)
    orphan$profile.antibio <- as.character(orphan$profile.antibio)
  }
  
  return(list(resolved=res, unresolved=orphan))
  
}




distance.from.ID<- function(from, to, nw=mat.day, dt, dur=0, lag=7) {
  # which minimal distance "to" and "from"
  # from: ego
  # to: alter
  # nw: contact matrix
  # dt: date
  # dur: minimal duration of contact (secondes)
  # lag: number of aggregated days 

  
  # Contact network Extraction 
  dt <- as.Date(dt)
  diff.dt <- difftime(dt, nw$day, units="day")
  tmp <- nw[diff.dt > 0 & diff.dt <= lag & as.numeric(as.character(nw$length)) >= dur, ]
  nw.bak <- tmp #backup for specific case 
  
  # Contact matrix is symetric 
  tmp <- unsymetrize.contact.matrix(tmp)
  
  if (dim(tmp)[1] == 0) {warning(paste("No network to return for date ", dt, ".\n", sep="")); return(NULL)}
  
  # aggregation of contacts
  tmp <- aggregate(tmp$length, by=list(tmp$from, tmp$to), sum)
  names(tmp) <- c("from", "to", "length")
  network.size <- nlevels(tmp$from)
  
  tmp$from <- as.character(tmp$from)
  tmp$to <- as.character(tmp$to)
  IDs.list <- from
  
  # IDs list
  l <- 0
  found <- FALSE #flag for the detection of the interesting neighboor
  full <- FALSE #flag all the network
  old.length <- 0
  while (full == FALSE & found == FALSE) {
    # neighboor list 
    IDs.list = unique(c(IDs.list, unique(c(tmp$from[tmp$to %in% IDs.list], tmp$to[tmp$from %in% IDs.list]))))
    if (to %in% IDs.list) found <- TRUE
    if (length(IDs.list) == length(unique(c(tmp$from, tmp$to))) | old.length == length(IDs.list)) full <- TRUE
    l <- l+1
    old.length <- length(IDs.list)
  }
  
  # neighboor not found
  if (found == FALSE) {warning(paste("ID ", to, " cannot be linked to ID ", from, " on this network (dt=", dt, ", dur=", dur, ", lag=", lag,").\n", sep="")); return(NULL)}
  
  # neighboor found
  else {
    # searching for intermediaries
    if (l==2) {
      tmp <- nw[diff.dt > 0 & diff.dt <= lag & nw$length >= dur & ((as.character(nw$from) %in% c(from, to)) | (as.character(nw$to) %in% c(from, to))), ]
      
      intermediates.from <- unique(c(as.character(tmp$from[as.character(tmp$to) == from]), as.character(tmp$to[as.character(tmp$from) == from])))
      intermediates.to <- unique(c(as.character(tmp$to[as.character(tmp$from) == to]), as.character(tmp$from[as.character(tmp$to) == to])))
      intermediates <- intersect(intermediates.from, intermediates.to)
      
      tmp <- tmp[(as.character(tmp$from) == from & as.character(tmp$to) %in% intermediates) |
                   (as.character(tmp$to) == from & as.character(tmp$from) %in% intermediates) |
                   (as.character(tmp$from) == to & as.character(tmp$to) %in% intermediates) |
                   (as.character(tmp$to) == to & as.character(tmp$from) %in% intermediates), ]
      
     
      return(c(l, paste(unique(intermediates), collapse=";")))
      
    }else if (l==3) {
      tmp <- nw[diff.dt > 0 & diff.dt <= lag & nw$length >= dur & ((as.character(nw$from) %in% c(from, to)) | (as.character(nw$to) %in% c(from, to))), ]
      
      intermediates.from <- unique(c(as.character(tmp$from[as.character(tmp$to) == from]), as.character(tmp$to[as.character(tmp$from) == from])))
      intermediates.to <- unique(c(as.character(tmp$to[as.character(tmp$from) == to]), as.character(tmp$from[as.character(tmp$to) == to])))
      tmp_inter<-nw[diff.dt > 0 & diff.dt <= lag & nw$length >= dur & ((as.character(nw$from) %in% intermediates.from) & (as.character(nw$to) %in% intermediates.to)),]
      intermediates <- unique(c(as.character(tmp_inter$from),as.character(tmp_inter$to)))
      
      tmp <- nw[diff.dt > 0 & diff.dt <= lag & nw$length >= dur & ((as.character(nw$from) == from & as.character(nw$to) %in% intermediates) |
                                                                     (as.character(nw$to) == from & as.character(nw$from) %in% intermediates) |
                                                                     (as.character(nw$from) == to & as.character(nw$to) %in% intermediates) |
                                                                     (as.character(nw$to) == to & as.character(nw$from) %in% intermediates)|
                                                                     (as.character(nw$from) %in% intermediates & as.character(nw$to)%in% intermediates)), ]
      
      
      return(c(l, paste(unique(intermediates), collapse=";")))
      
    }
    else return(c(l, NA))
  }
}


unsymetrize.contact.matrix <- function(x) {
  # Keep just A->B contacts
  #
  # x: contact matrix
  
  x$from <- factor(as.character(x$from), levels=sort(unique(c(as.character(x$from), as.character(x$to)))))
  x$to <- factor(as.character(x$to), levels=sort(unique(c(as.character(x$from), as.character(x$to)))))
  

  if ("date_posix" %in% colnames(x)) {
    x$from.num <- as.numeric(x$from)
    x$to.num <- as.numeric(x$to)
    x$min.from.to <- pmin(x$from.num, x$to.num)
    x$max.from.to <- pmax(x$from.num, x$to.num)
    dupl <- duplicated(data.frame(date=x$date_posix, min.from.to=x$min.from.to, max.from.to=x$max.from.to))
    res <- x[!dupl, -match(c("from.num", "to.num", "min.from.to", "max.from.to"), colnames(x))]
  }
  else {

    res1 <- x[(as.numeric(x$from) > as.numeric(x$to)), ]
    res2 <- x[(as.numeric(x$from) < as.numeric(x$to)), ]
    
    if (dim(res1)[1] >= dim(res2)[1]) res <- res1 else res <- res2
  }

  res <- res[order(res$day), ]
  
  if (dim(res)[1] > 0) row.names(res) <- 1:dim(res)[1]
  
  return(res)
}


##################################################
##  observation and permutation main functions  ##
##################################################
observations<-function(level,what, mat.day, col.entero){
  candidate.infector<-list()
  infector<-list()
  for(i in seq(7,level,by=7)){
    if(i==7){
      candidate.infector[[as.character(i)]]<-find.same.germ(col.entero[as.Date(as.character(col.entero$date_prl)) > min(as.Date(as.character(mat.day$day)) + 7), ], prlvt=entero.all, lag=i, what=what,delta.profile=0, verbose=TRUE)
      infector[[as.character(i)]] <- find.closest.infector(candidate.infector[[as.character(i)]], dur=0, lag=i, verbose=TRUE, nw=mat.day, keepall=TRUE)
    }else{
      candidate.infector[[as.character(i)]]<- find.same.germ(unique(rbind(candidate.infector[[as.character(i-7)]]$notfound, infector[[as.character(i-7)]]$unresolved)), prlvt=entero.all, lag=i, what=what,delta.profile=0, verbose=TRUE) 
      infector[[as.character(i)]]<- find.closest.infector(candidate.infector[[as.character(i)]], dur=0, lag=i, verbose=TRUE, nw=mat.day, keepall=TRUE)
    }
    
  }
  
  col.entero$uniqueID<-1:dim(col.entero)[1]
  
  for(i in seq(7,level,by=7)){
    if(is.null(dim(infector[[as.character(i)]]$resolved[1]))){next}
    for (j in 1:(dim(infector[[as.character(i)]]$resolved)[1])) {
      infector[[as.character(i)]]$resolved$group[j] <- col.entero$uniqueID[as.character(col.entero$calc_ident) == as.character(infector[[as.character(i)]]$resolved$case[j])
                                                                           & as.Date(as.character(col.entero$date_prl)) == as.Date(as.character(infector[[as.character(i)]]$resolved$case_date_prl[j]))
                                                                           & as.numeric(col.entero$BLSE) == as.numeric(infector[[as.character(i)]]$resolved$BLSE[j])
                                                                           
                                                                           & as.character(col.entero$profile.antibio) == as.character(infector[[as.character(i)]]$resolved$profile.antibio.cas[j])
                                                                           ]
    }
  }
  return(list(candidate.infector,infector,col.entero))
}

permutations<-function(level,what,nb_perm, mat.day, col.entero){
  entero.perm <- NULL
  entero.perm.dist <- list()
  entero.perm.inf<-list()
  for (i in 1:nb_perm) {
    print(paste("permutation:",i))
    
    candidate.infector.sim<-list()
    infector.sim<-list()
    for(semaine in seq(7,level,by=7)){
      print(paste("permutation:",i," semaine:",semaine))
      if(semaine==7){
        p<-1
        candidate.infector.sim[[as.character(semaine)]]<-find.same.germ(col.entero[as.Date(as.character(col.entero$date_prl)) > min(as.Date(as.character(mat.day$day)) + 7), ], prlvt=entero.all, lag=semaine, what=what,delta.profile=0, shuffle=TRUE, pass=p, b=level,verbose=TRUE)
        infector.sim[[as.character(semaine)]] <- find.closest.infector(candidate.infector.sim[[as.character(semaine)]], dur=0, lag=7, verbose=TRUE, keepall=TRUE, nw=mat.day)
      }else{
        p<-p+1
        candidate.infector.sim[[as.character(semaine)]]<- find.same.germ(unique(rbind(candidate.infector.sim[[as.character(semaine-7)]]$notfound, infector.sim[[as.character(semaine-7)]]$unresolved)), prlvt=candidate.infector.sim[[as.character(7)]]$prlvt, lag=semaine, what=what,delta.profile=0, shuffle=TRUE, pass=p,b=level, verbose=TRUE)
        infector.sim[[as.character(semaine)]]<- find.closest.infector(candidate.infector.sim[[as.character(semaine)]], dur=0, lag=semaine, keepall=TRUE, verbose=TRUE, nw=mat.day)
      }
      
    }
    wt<-0
    for(w in seq(7,level,by=7)){
      wt<-wt+1
      if(!is.null(infector.sim[[as.character(w)]]$resolved)){
        #print(w)
        infector.sim[[as.character(w)]]$resolved$week.to.resolve <- wt
      }
    }
    
    r<-NULL
    e.inter<-unlist(lapply(infector.sim,function(x){cbind(r,length(unique(x$resolved$group)))}))
    entero.perm <- rbind(entero.perm,e.inter)
    
    #UniqueID
    
    for(se in seq(7,level,by=7)){
      if(is.null(dim(infector.sim[[as.character(se)]]$resolved)[1])){next}
      for (j in 1:(dim(infector.sim[[as.character(se)]]$resolved)[1])) {
        infector.sim[[as.character(se)]]$resolved$group[j] <- col.entero$uniqueID[as.character(col.entero$calc_ident) == as.character(infector.sim[[as.character(se)]]$resolved$case[j])
                                                                                  & as.Date(as.character(col.entero$date_prl)) == as.Date(as.character(infector.sim[[as.character(se)]]$resolved$case_date_prl[j]))
                                                                                  & as.numeric(col.entero$BLSE) == as.numeric(infector.sim[[as.character(se)]]$resolved$BLSE[j])
                                                                                  
                                                                                  & as.character(col.entero$profile.antibio) == as.character(infector.sim[[as.character(se)]]$resolved$profile.antibio.cas[j])
                                                                                  ]
      }
    }
    
    res<-NULL
    test<-lapply(infector.sim,function(x){return(rbind(res,x$resolved[, c("case","case_date_prl","distance", "intermediate","group", "week.to.resolve")]))})
    tmp<-do.call(rbind, test)
    
    entero.perm.dist[[i]] <- tmp
  }
  return(list(entero.perm.dist,entero.perm))
}

