############################################
## ---- ---- ---- ---- ---- ---- ---- --- ##
##         ESBL code permutation          ##             
##             Audrey DUVAL               ##
##        Last Updated 07 01 2019         ##
## ---- ---- ---- ---- ---- ---- ---- --- ##
############################################
rm(list=ls())
library(ggplot2)
library(reshape2)
library(grid)
library(reshape2)
library(gridExtra)
library(reshape2)

# Import 
source("permutation_functions_light.R")
load("admission.raw.RData")
load("mat.day.RData")
load("entero_dataFrame.RData")

#--------------------- INCIDENT-COLONIZATION EPISODES -------------------#
#parameters 
# what<-"Escherichia coli"
what<-"Klebsiella pneumoniae"
level<-28

episodes<-select.episodes(what=what,with = "BLSE", lissage=FALSE)
col.entero <- episodes[order(as.character(episodes$calc_ident), episodes$date_prl_posix),]
col.entero<-SupSimilar(col.entero)


#-------------------- OBSERVATIONS AND PERMUTATIONS ---------------------#
# OBSERVATIONS
obs<-observations(level,what, mat.day, col.entero)
candidate.infector<-obs[[1]]
infector<-obs[[2]]
col.entero<-obs[[3]]

# PERMUTATIONS
nb_perm<-200
perm<-permutations(level,what,nb_perm, mat.day, col.entero)
entero.perm.dist<-perm[[1]]
entero.perm<-perm[[2]]

####################################################### RESULTS #######################################################
#--------------------------------------------------- DISTRIBUTIONS ---------------------------------------------------#

entero.sim <- lapply(entero.perm.dist, function(x) {t <- aggregate(x$distance, by=list(x$case,x$case_date_prl,x$group), min); names(t) <- c("case","case_date_prl","group", "distance"); return(t)})
groups <- sort(unique(unlist(lapply(entero.sim, function(x) x$group))))
sim <- data.frame(case=NA,case_date_prl=NA,group=groups, distance=NA)
for (g in groups) {
  tmp <- NULL
  indiv<-NULL
  date<-NULL
  for (l in 1:(length(entero.sim))) {
    if (!is.na(entero.sim[[l]]$distance[match(g, entero.sim[[l]]$group)])) {
      tmp <- c(tmp, entero.sim[[l]]$distance[match(g, entero.sim[[l]]$group)])
      indiv<-c(indiv,as.character(entero.sim[[l]]$case[match(g, entero.sim[[l]]$group)]))
      date<-c(date,as.Date(entero.sim[[l]]$case_date_prl[match(g, entero.sim[[l]]$group)],origin="1970-01-01"))
    }
  }
  sim$distance[match(g, sim$group)] <- mean(tmp)
  sim$case[match(g, sim$group)]<-unique(indiv)
  sim$case_date_prl[match(g, sim$group)]<-unique(date)
}
entero.sim <- sim
rm(groups, g, l, sim)

entero.sim$case_date_prl<-as.Date(entero.sim$case_date_prl,origin="1970-01-01")


#----------------------- TEST --------------------#
entero.obs<-NULL
for(i in seq(7,level, by=7)){
  entero.obs <- rbind(entero.obs,infector[[as.character(i)]]$resolved[, c("distance", "group")])
}
entero.obs <- aggregate(entero.obs$distance, by=list(entero.obs$group), min)
names(entero.obs) <- c("group", "distance")
entero.sim <- entero.sim[entero.sim$group %in% entero.obs$group, ]
entero.sim <- entero.sim[order(entero.sim$group), ]

#wilcox test
mat.day.mindist.wilcox <- wilcox.test(entero.sim$distance, entero.obs$distance, paired=T)

#plot
dist.obs <- table(factor(entero.obs$distance, levels=1:5))
names(dist.obs) <- c(1:4, "5+")
dist.obs <- dist.obs/sum(dist.obs)
barplot(dist.obs*100)

tmp.complete <- list()
tmp.complete <- lapply(entero.perm.dist, function(x) table(tapply(x$distance, x$group, min))) 
tmp.complete <- lapply(tmp.complete, function(x) x/sum(x))
tmp <- NULL
for (i in 1:5) {
  tmp.i <- NULL
  for (j in 1:length(tmp.complete)) {
    if(length(tmp.complete[[j]][which(names(tmp.complete[[j]]) == i)])==0){
      tmp.i <- c(tmp.i,0)
    }else{
      tmp.i <- c(tmp.i, tmp.complete[[j]][which(names(tmp.complete[[j]]) == i)]) 
    }
  }
  tmp <- c(tmp, mean(tmp.i))
}
dist.shuffle<-tmp
barplot(rbind(dist.obs, dist.shuffle)*100, beside=T, xlab="Distance", ylab="Percentage", 
        cex.lab=1.5, cex.axis=1.5, cex.names=1.5, ylim=c(0,100),col=c("grey","black"))
legend("topright", c("observation","permutation"), fill = c("grey","black"), bty = "n")
